/*
 *	help.c - help once the game is going
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	OneLineHelp()		: basic 1-line help during play
 *	FullScreenHelp()	: full screen help
 *	ShowFile()			: show a file
 */

#include "Text.h"
#include "eek.h"

static char *sym_help[] = {
	"You (as in, 'It is I'.)",
	"Braindead Sitzbath (robotlike)",
	"Fence or other obstacle (dangerous)",
	"Junk heap (beware of high voltage)",
	"Berserker (death ship)",
	"Berserker trap",
	"Juggernaut (almost unstoppable)",
	"Black Hole (beware the event horizon!)",
	"I've never seen one of those.",
};

#define SHOW_HELP(SYM,STR) {\
	wmove(helpline, 0, 0);\
	wclrtoeol(helpline);\
	PUT_CH_SO(helpline, 0, 1, SYM);\
	wprintw(helpline, " : ");\
	wprintw(helpline, STR);\
	wprintw(helpline, " [enter symbol or ENTER (done)] ");\
}

#define MAIN_PROMPT() {\
	wmove(helpline, 0, 0);\
	wclrtoeol(helpline);\
	wprintw(helpline, "Enter symbol [or '?' (more help) or ENTER (done)] ");\
}

/*
 *	OneLineHelp()		: basic 1-line help during play
 */

void
OneLineHelp(x, y)
{
	char response;
/*
 *	ask puppeteer what they need help with
 */
	if (! helpline)
		helpline = newwin(1, COLS, (LINES - 1), 0);
	MAIN_PROMPT();
	do {
		wrefresh(helpline);
		GETCH(response);
		switch (response) {
			case HUMANOID :
				SHOW_HELP(HUMANOID, sym_help[0]);
				break;
			case ROBOT :
				SHOW_HELP(ROBOT, sym_help[1]);
				break;
			case FENCE :
				SHOW_HELP(FENCE, sym_help[2]);
				break;
			case JUNK :
				SHOW_HELP(JUNK, sym_help[3]);
				break;
			case BERSERKER :
				SHOW_HELP(BERSERKER, sym_help[4]);
				break;
			case TRAP :
				SHOW_HELP(TRAP, sym_help[5]);
				break;
			case JUGGERNAUT :
				SHOW_HELP(JUGGERNAUT, sym_help[6]);
				break;
			case BLACK_HOLE :
				SHOW_HELP(BLACK_HOLE, sym_help[7]);
				break;
			case '?' :
				FullScreenHelp();
				ForceRefreshBF(x, y);
				touchwin(helpline);
				break;
			case MENU_DONE :
				goto BOOGIE;
			default :
				SHOW_HELP(response, sym_help[8]);
				break;
		}
	} while (1);
BOOGIE:
	wmove(helpline, 0, 0);
	wclrtoeol(helpline);
	wrefresh(helpline);
	return;
}

static void
	EekObjects(),
	EekDetObj(),
	EekStrategy(),
	EekTactics();

static MenuItem items[] = {
	{MenuItemText,   F,F,F, "Do you wish to:", EOS },
	{MenuItemPrompt, F,T,F, "   see all eek objects    (O),", 'O', EekObjects },
	{MenuItemPrompt, F,T,F, "   details of eek objects (D),", 'D', EekDetObj },
	{MenuItemPrompt, F,T,F, "   ponder eek strategy    (S),", 'S', EekStrategy },
	{MenuItemPrompt, F,T,F, "   learn eek tactics      (T),", 'T', EekTactics },
	{MenuItemText,   F,F,F, "   or quit help           (Q)?", EOS },
	{MenuItemText,   T,F,F, "Enter O, D, S, T, or Q ", EOS },
};

Menu full_screen_help_menu = {
	MenuBoxed,
	JustifyHCenter,
	JustifyVCenter,
	items,
	NumItems(items),
};

static char file[256] = { EOS };

/*
 *	FullScreenHelp()	: full screen help
 */
void
FullScreenHelp()
{
/*
 *	Run the menus.  When done, clean up & leave.
 */
	SimpleMenu(&full_screen_help_menu);
}



static void
EekObjects()
{
	ShowFile("eek_what");
}


static void
EekDetObj()
{
	ShowFile("eek_obj_long");
}


static void
EekStrategy()
{
	ShowFile("eek_strategy");
}


static void
EekTactics()
{
	ShowFile("eek_tactics");
}


void
ShowFile(file)
char *file;
{
	static char dir[256];

	strcpy(dir, eek_dir);
	strcat(dir, "/");
	SMTEXT(dir, file);
}
