/*
 *	quantum.c - quantum physics routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 * InGravityWell()		: ret. 1 if obj at (x, y) is in a gravity well, else 0
 *	CreateBlackHoles()	: create black holes
 *	CreateBlackHole()		: create & place black
 */

#include "eek.h"
#include "Random.h"


/*
 * InGravityWell()		: ret. 1 if obj at (x, y) is in a gravity well, else 0
 */
int
InGravityWell(x, y)
{
	if (black_hole &&
			(abs(x - bh_x) <= black_hole && abs(y - bh_y) <= black_hole))
		return 1;
	else
		return 0;
}


/*
 *	CreateBlackHoles()	: create black holes
 */
void
CreateBlackHoles()
{
	CreateBlackHole();
}


/*
 *	CreateBlackHole()		: create & place black hole
 */
void
CreateBlackHole()
{
	int x, y;
/*
 *	Pick a size for a black hole, pseudo-sorta-randomly.
 */
	if (Random(1 + ((level % 4) ? 0 : (level / 4))))
		black_hole = BIG_BH;
	else if (Random(1 + ((level % 2) ? 0 : (level / 2))))
		black_hole = LITTLE_BH;
	else
		black_hole = NO_BH;
/*
 *	Can't have anything in the black hole's gravity well!
 * Create it at least a couple of gridloc's away from the edge
 * of the universe.
 */
	if (black_hole) {
		is_worm_hole = Random(5) ? 0 : 1;
		do {
			x = MungedRandom(NX-6) + 3;
			y = MungedRandom(NY-6) + 3;
		} while (grid[x][y]);
		bh_x = x;
		bh_y = y;
		grid[x][y] = BLACK_HOLE;
		for (x = 1; x < NX-1; x++) {
			for (y = 1; y < NY-1; y++) {
				if (InGravityWell(x, y)) {
					if (grid[x][y] != BLACK_HOLE) {
						if (grid[x][y] == ROBOT)
							nrobots--;
						else if (grid[x][y] == BERSERKER)
							nberserk--;
						else if (grid[x][y] == FENCE)
							nfences--;
						grid[x][y] = EOS;
					}
				}
			}
		}
	} else {
		bh_x = bh_y = 0;
	}
}
