/*
 *	score.c : scoring routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	ScoreMove()		: compute score at end of move
 *	ScoreLevel()	: compute score at end of each level or game over
 */

#include "eek.h"

static u_long lvl_score = 0;

/*
 *	ScoreMove()	: compute score at end of move
 *
 *		Don't update it unless it's changed.
 */

void
ScoreMove()
{
	/* register */ int x, y;
	/* register */ int lvl_robots = nrobots;
	/* register */ int lvl_ber = nberserk;
	int lvl_rob_pts = 0;
	int lvl_ber_pts = 0;
/*
 *	count down from the # started with this level, 1 per
 *	each left alive.  what's left is the # killed.
 */
	for (y = NY-1; y > 0; y--)
		for (x = NX-1; x > 0; x--)
			if (grid[x][y] == ROBOT)
				lvl_robots--;
			else if (grid[x][y] == BERSERKER)
				lvl_ber--;
				
	if (lvl_robots != nrobots || lvl_ber != nberserk) {
		lvl_rob_pts = (u_long) (lvl_robots) * (u_long) rob_pts;
		lvl_ber_pts = (u_long) (lvl_ber) * (u_long) ber_pts;
		lvl_score = (u_long) lvl_rob_pts + (u_long) lvl_ber_pts;
		lvl_score *= norm_fac;
		UpdateScorewin(score + lvl_score);
	}
}




/*
 *	ScoreLevel()	: compute score at end of each level or game over
 *
 *		if (status == ALIVE)
 *			end of level - add level bonus
 *		else
 *			game over, no bonus
 */

void
ScoreLevel(status)

int status;
{
	/* register */ int moves_edge = 0;
	int fast_bonus = 0;

	if (status == ALIVE) {
		moves_edge = (25 / norm_fac) + (level * (3 + nberserk));
		if (moves < moves_edge) {
			moves_edge -= moves;
			fast_bonus += ((level < 10) ?
				(moves_edge * 25) :
					(moves_edge * 3 * level));
		} else {
			moves_edge = 0;
		}
		lvl_score += fast_bonus;
	}
	score += lvl_score;
	if (status == ALIVE)
		UpdateScorewin(score);
	lvl_score = 0;
}
