/*
 *	signals.c	- signal handlers
 *
 *	DieGracefully()			: exit cleanly
 *	DieImmediately()			: exit with minimal cleanup
 *	ItsImpossible()			: should never happen
 *	RedrawScreen()				: redraw the screen
 *	SetupSignalHandlers()	: setup all rational signal handlers
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-9 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 */

#if defined(linux)
#	define __KERNEL__
#	include <asm/signal.h>
#	undef __KERNEL__
#	define Sigcontext sigcontext_struct
#include <signal.h>
#else
#include <signal.h>
#	define Sigcontext sigcontext
#endif
#include "eek.h"

#if defined(BSF) || defined(NeXT)
#	define SignalHandler int
#elif defined(SySV)
#	define SignalHandler void
#elif defined(__STDC__)
#	define SignalHandler void
#else
#	define SignalHandler int
#endif

typedef SignalHandler (*SignalFunc)();

static SignalHandler
	DieGracefully PL_(( int sig, int code, struct Sigcontext *scp)),
	DieImmediately PL_(( int, int, struct Sigcontext *)),
	ItsImpossible PL_(( int, int, struct Sigcontext *)),
	RedrawScreen PL_(( int, int, struct Sigcontext *));

typedef enum {
	ReSc,
	DieG,
	DieI,
	What,
	None,
	Last,
} SignalType;

static SignalFunc signal_functions[] = {
	RedrawScreen,		/* ReSc */
	DieGracefully,	/* DieG */
	DieImmediately,	/* DieI */
	ItsImpossible,	/* What */
	NULL,					/* None */
};


static SignalType sig_map[] = {
#if defined(Next) || defined(ultrix)
		ReSc, DieG, DieI, None, None, None, None, None, None, None,
		None, None, DieI, What, DieG, DieI, None, None, None, What,
		None, None, None, DieG, DieG, What, What, None, What, None,
		None, Last
#elif defined(_AIX)
		ReSc, DieG, DieI, None, None, None, None, None, None, None,
		None, None, DieI, What, DieG, DieI, None, None, None, What,
		None, None, None, DieG, DieG, None, None, None, What, None,
		None, What, DieI, What, None, What, What, What, What, What,
		What, What, What, What, What, What, What, What, What, What,
		What, What, What, What, What, What, What, What, What, None,
		None, None, None, Last
#elif defined(hpux)
		ReSc, DieG, DieI, None, None, None, None, None, None, None,
		None, None, DieI, What, DieG, What, What, None, What, What,
		What, None, None, None, None, None, None, None, DieI, None,
		Last
#elif defined(SYSTYPE_SYSV)
		ReSc, DieG, DieI, None, None, None, None, None, None, None,
		None, None, DieI, What, DieG, None, None, What, What, None,
		DieI, None, None, None, None, None, None, What, What, None,
		None, Last
#else /* who knows?  Just handle the first 15... */
		ReSc, DieG, DieI, None, None, None, None, None, None, None,
		None, None, DieI, What, DieG, Last
#endif
};


/*
 *	DieGracefully ()	: exit cleanly
 */

static SignalHandler
DieGracefully(sig, code, scp)
int sig, code;
struct Sigcontext *scp;
{
	signal(sig, SIG_DFL);
	LevelExit(WIMPED_OUT);
}



/*
 *	DieImmediately()	: exit with minimal cleanup
 */

static SignalHandler
DieImmediately(sig, code, scp)
int sig, code;
struct Sigcontext *scp;
{
	signal(sig, SIG_DFL);
	LevelExit(ESCAPED);
}



/*
 *	RedrawScreen()	: redraw the screen
 */

static SignalHandler
RedrawScreen(sig, code, scp)
int sig, code;
struct Sigcontext *scp;
{
	signal(sig, RedrawScreen);
	ForceRefresh();
}



/*
 *	ItsImpossible()	: whine
 */

static SignalHandler
ItsImpossible(sig, code, scp)
int sig, code;
struct Sigcontext *scp;
{
	signal(sig, ItsImpossible);
	fprintf(stderr, "Eek signal violation: signal %d caught\n", sig);
	fflush(stderr);
}



/*
 *	SetupSignalHandlers()	: setup signal handlers
 */
void SetupSignalHandlers()
{
	/* register */ int s;

	for (s = 0; sig_map[s] != Last; s++)
		if (sig_map[s] != None)
			signal(s+1, signal_functions[sig_map[s]]);
}
