/*
 *	teleport.c - teleportation routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-8 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	Teleport()				: teleport to a random location
 *	InitTeleportLevel()	: initialize level teleportation params
 */

#include "eek.h"
#include "Random.h"

/*
 *	Teleport(int *px, *py, worm_hole) : teleport to a random location
 *
 *	The current player x & y are pointed to by px & py.
 *	The new locations will be returned there as well.
 * If we are going through a worm hole, try extra hard
 * to be nice.
 */


void
Teleport(px, py, worm_hole)
int *px, *py;
int worm_hole;
{
	/* register */ int nx, ny;		/* new x, y */
	/* register */ int max_tries = 0;
	int benice, tnice = -1;

/*
 *	The more we have to teleport because we are trapped,
 *	the better the odds.  tnice starts at -1 to account
 *	for the humanoid (less testing).
 */
	for (nx = *px - 1; nx < (*px + 1); nx++)
   		for (ny = *py - 1; ny < (*py + 1); ny++)
			if (grid[nx][ny] != EOS && grid[nx][ny] != '|')
				tnice++;
	
/*
 *	benice is whether to be nice & not drop them right next to
 *	something. 1 time out of N we won't bother to check. So
 *	the odds are still favorable for teleporting.  Even if we
 *	are being nice, if we haven't found somewhere by 10,000
 *	tries, give up & stick Fred wherever...
 */
	benice = worm_hole? 1 : Random(nice_lvl + tnice);
TRY_AGAIN:
	do {
		nx = MungedRandom(NX - 2) + 1;
		ny = MungedRandom(NY - 2) + 1;
	} while (grid[nx][ny] != EOS);
	if (benice && (++max_tries < 10000)) {
		/* register */ int tx, ty;		/* temp x, y */
		int xmin, ymin;
		/* register */ int xmax, ymax;
		int local_nd;				/* local version of nice_dist */

		if (InGravityWell(nx, ny))
			goto TRY_AGAIN;
		local_nd = MIN(nice_dist, bmaxmvz);
		xmin = MAX(1, nx - local_nd);
		xmax = MIN(NX - 1, nx + local_nd);
		ymin = MAX(1, ny - local_nd);
		ymax = MIN(NY - 1, ny + local_nd);
		for (tx = xmin; tx <= xmax; tx++) {
			for (ty = ymin; ty <= ymax; ty++) {
				if (((abs(tx - nx) > 1 || abs(ty - ny) > 1) &&
						grid[tx][ty] == BERSERKER) ||
						((abs(tx - nx) == 1 || abs(ty - ny) == 1) &&
						(grid[tx][ty] == ROBOT || grid[tx][ty] == JUGGERNAUT)))
							goto TRY_AGAIN;
			}
		}
	}
	tele_tries = max_tries;
	*px = nx;
	*py = ny;
}


/*
 *	InitTeleportLevel()	: initialize level teleportation params
 */
void
InitTeleportLevel()
{
	nice_dist = (level / 10) + 1;
	nice_lvl += nice_dist;
}
