/*
 *	wrapup.c - wrapup play & game routines
 *
 *	Copyright 1988, 1990, Miles ONeal, Austin, TX, USA. All rights reserved.
 *	Copyright 1993, 1997-9 Roadkills-R-Us, Austin, TX, USA. All rights reserved.
 *
 *	Munch()		: kill player, set death type & split
 *	LevelExit()	: tell user why level is over, split if dead or quit
 *	EndGame()	: save info to score file & display it
 *	Cleanup()	: erase screen & shut down curses
 */

#include "eek.h"
#include "Random.h"
#include <time.h>

/*
 *	Munch()		: kill player, set death type & split
 */

static char *death_noise[] = {
	"*ARRGH*",
	"*PLOTZ*",
	"*SPLAT*",
	"*BRZZT*",
	"*GAACK*",
	"*SMASH*",
	"*DEATH*",
};

static ndn = sizeof(death_noise) / sizeof(char *);	/* # of death noises */

static char *surv_expl[] = {
	"Not quite being corporeal, you survived!",
	"A Karres witch saved you via the Eggar route!",
	"A vatch saved you.  Why?  A scary thought...",
	"A tinkle of witch notes sounds in your head.",
	"To be or not to be - for the moment you be.",
	"What happened?  Where am I?",
	"This is not my beautiful body!",
};

static nse = sizeof(surv_expl) / sizeof(char *);	/* # of survival explanations */

static void Cleanup();

void
Munch(
#ifdef __STDC__
int x, int y, chtype type
)
#else
x, y, type)
int x, y;
chtype type;
#endif

{
/*
 *	if in training, never, ever smite Fred
 */
	if (training) {
		PUT_STR(field, NY - 1, 0, "Amazingly, you survived!");
		wrefresh(field);
#ifdef NORMAL
		sleep(1);
#endif
		need_box = 1;
	} else if (score < MIN_SURV_SCORE) {
		PUT_STR(field, NY - 1, 0, surv_expl[Random(nse)]);
		wrefresh(field);
		sleep(2);
		need_box = 1;
		BOX_FIELD();
		PUT_CH_SO(field, y, x, HUMANOID);
		wrefresh(field);
	} else if (lives > 0) {
		lives--;
		PUT_STR(field, NY - 1, 0, "You feel like a new humanoid!");
		wrefresh(field);
		sleep(2);
		need_box = 1;
		BOX_FIELD();
		PUT_CH_SO(field, y, x, HUMANOID);
		wrefresh(field);
	} else {
/*
 *		otherwise, make nasty death sounds,
 *		dispose of the body & split
 */
		if (x > NX-7)
			x = NX-8;
		PUT_STR(field, y, x, death_noise[Random(ndn)]);
		wrefresh(field);
		death_type = type;
		sleep(1);
		LevelExit(KILLED);
	}
}

/*
 *	LevelExit()	: tell user why level is over, split if dead or quit
 */

void
LevelExit(status)
int status;
{
	wmove(field, NY - 1, 0);
	switch (status) {
		case SURVIVED :
			wprintw(field, "You have defeated all the robots!");
			wrefresh(field);
			sleep(2);
			need_box = 1;
			break;
		case KILLED :
			wprintw(field, "You have been reduced to lifeless carbon compounds!");
			wrefresh(field);
			sleep(2);
			/* FALL THROUGH! */
		case WIMPED_OUT :
			Cleanup();
			ScoreLevel(DEAD);
			exit(EndGame(status));
			break;
		case ESCAPED :
			Cleanup();
			exit(0);
			break;
	}
}

/*
 *	Cleanup()	: erase screen & shut down curses
 */

static
void
Cleanup()
{
	extern Menu full_screen_help_menu;

	if (helpline) {
		werase(helpline);
		wrefresh(helpline);
	}
	if (testwin) {
		werase(testwin);
		wrefresh(testwin);
	}
	if (scorewin) {
		werase(scorewin);
		wrefresh(scorewin);
	}
	if (field) {
		werase(field);
		wrefresh(field);
	}
	if (MenusRunning()) {
		DestroyMenu(&full_screen_help_menu);
		EndMenus(&main_menu);
	}
	if (stdscr) {
		erase();
		refresh();
		endwin();
	}
}

/*
 *	EndGame()	: save info to score file & display it
 */

#if defined(USE_CTIME)
	static char *score_h0 = { "                                 SCORES\n" };
	static char *score_h1 =
		{ "Score    Lvl          Who   &   What                     When\n" };
	static char *score_h2 =
		{ "-------- --- --------------------------------- ------------------------\n" };
	static char *score_fmt = { "%8lu %3d %-8.8s %-24.24s %-24s\n" };
#else /* USE_CTIME */
	static char *score_h0 = { "                            SCORES\n" };
	static char *score_h1 =
		{ "Score    Lvl          Who   &   What                When\n" };
	static char *score_h2 =
		{ "-------- --- --------------------------------- --------------\n" };
	static char *score_fmt = { "%8lu %3d %-8.8s %-24.24s %-14s\n" };
#endif /* USE_CTIME */

int
EndGame(q)
int q;		/* 2 if quit, else died */
{
	extern char *ctime();
	FILE *fp;
	char *who, *what;

#if defined(USE_CTIME)
	char *when;
#else /* USE_CTIME */
	char when[19];
	struct tm *twhen;
#endif /* USE_CTIME */

	long time(), t;
	char new_score[512];		/* new score file path */
	char eek_scores[512];	/* old score file path */
	char command[1024];		/* CLI to integrate new score & display scores */

	if (score == 0) {
		if (q == 2) {
			printf("You SKIPPED OUT with NO POINTS!\n\nWEASEL!\n\n");
		} else {
			printf("You died with no points!\n\nTry again!\n\n");
		}
		goto BOOGIE;
	}

	t = time((long *) 0);

#if defined(USE_CTIME)
	when = ctime(&t);
	when[24] = EOS;
#else /* USE_CTIME */
	twhen = localtime(&t);
        sprintf (&(when[0]), "%2.2d/%2.2d/%2.2d %2.2d:%2.2d",
		twhen->tm_year, twhen->tm_mon, twhen->tm_wday,
		twhen->tm_hour, twhen->tm_min);
#endif /* USE_CTIME */

	who = user_name;
	if (q == 1) {
		switch (death_type) {
			case ROBOT:
				what = robot_obit;
				break;
			case JUNK:
				what = junk_obit;
				break;
			case FENCE:
				what = fence_obit;
				break;
			case TRAP:
				what = trap_obit;
				break;
			case JUGGERNAUT:
				what = juggernaut_obit;
				break;
			case BERSERKER:
				what = berserker_obit;
				break;
			case BLACK_HOLE:
				what = bh_obit;
				break;
			default:
				what = unknown_soldier;
				break;
		}
	} else {
		what = "retired";
	}
	printf("%s%s%s", score_h0, score_h1, score_h2);
	if (training || testing) {
		printf(score_fmt, score, level, who, what, when);
		if (testing) {
			printf("nrobots = %3d\n", nrobots);
			printf("rob_pts = %4d\n", rob_pts);
			printf("nberserk = %3d\n", nberserk);
			printf("bmaxmvz = %3d\n", bmaxmvz);
			printf("ber_pts = %7d\n", ber_pts);
			printf("max_traps = %3d\n", MAX_TRAPS);
			printf("nfences = %3d\n", nfences);
			printf("nice_lvl = %4d\n", nice_lvl);
			printf("nice_dist = %3d\n", nice_dist);
			printf("jug_likely = %3d\n", jug_likely);
			printf("black_hole = %3d\n", black_hole);
			printf("norm_fac = %5.3f\n", norm_fac);
			printf("tele_tries = %5d\n", tele_tries);
		}
	} else {
		strcpy(new_score, eek_dir);
		strcat(new_score, "/.new_score");
		if (!(fp = fopen(new_score, "w")))
			goto BOOGIE;
		fprintf(fp, score_fmt, score, level, who, what, when);
		fclose(fp);

		strcpy(eek_scores, eek_dir);
		strcat(eek_scores, "/.eek_scores");
		setuid(USER_UID);
		if (chmod(eek_scores, 0666))
			goto BOOGIE;
		umask(0000);
		strcpy(eek_scores, eek_dir);
		sprintf(command, "mv %s/.eek_scores %s/sqz;sort -rn +0 -3 %s/sqz|head -14>%s/tmp_scores; sort -rn %s/tmp_scores %s|tee %s/.eek_scores;rm -f %s/sqz %s %s/tmp_scores",
			eek_scores, eek_scores, eek_scores, eek_scores, eek_scores, new_score,
			eek_scores, eek_scores, new_score, eek_scores);
		if (!system(command)) {
			strcat(eek_scores, "/.eek_scores");
			chmod(eek_scores, 0444);
		}
	}
BOOGIE:
	chdir(orig_cwd);
	return 0;
}
